package projet_these.es;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.StringTokenizer;
import projet_these.et.Population;
import projet_these.et.IslandModel;
import java.io.FileNotFoundException;
import java.io.FileReader;
import projet_these.et.PopulationPoly1;
import projet_these.et.PopulationLocalBurst;
import projet_these.et.transElement;

/**
 * Implementaion of static methods to launch models from files
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 01/03/02
 */
public class InitIslanModelFromFile
{

    /**
     * Procedure of parameters initialization by the input file
     * @param model The starting model
     */
    public static void initModel(IslandModel model)
    {

        // Flux d'octets en lecture
        BufferedReader entree = null;

        // Chaine de caracteres marquee
        StringTokenizer st;

        // Exception de recherche d'un fichier
        FileNotFoundException exc;

        /* Intancie un tampon de lecture construit sur le fichier
                 texte d'entree en lecture avec un do while
                 de gestion des erreurs*/
        do {
            exc = null;
            try {
                entree = new BufferedReader(new FileReader(model.nomFic1));
            } catch (FileNotFoundException excp) {
                exc = excp;
                System.out.println("File " + model.nomFic1 + " not found\n" +
                                   "Enter a new input file name :");
                try {
                    model.nomFic1 = ReadStandardInput.readString();
                } catch (IOException e) {
                    System.out.println(e);
                }
            }
        } while (exc != null)
        ;

        // Instanciation des parametres separateur apres separateur
        try {
            // Saisie du model de contamination
            model.conta_mod = entree.readLine();

            // Saisie du nombre de generations
            model.tps = Integer.valueOf(entree.readLine()).intValue();

            // Saisie du nombre de populations
            model.nb_pop = Integer.valueOf(entree.readLine()).intValue();

            /* Saisie des differentes tailles de populations:
                         - dans le cas de tailles homogenes
                         - dans le cas de tailles heterogenes
             */
            model.N = new int[model.nb_pop];
            model.rep_tailles = entree.readLine();

            // Cas de tailles homogenes
            if (model.rep_tailles.equals("H")) {
                model.N[0] = Integer.valueOf(entree.readLine()).intValue();
                for (int i = 1; i < model.nb_pop; i++) {
                    model.N[i] = model.N[0];
                }
            }

            // Cas de tailles heterogenes
            if (model.rep_tailles.equals("NH")) {
                /* Dans le cas de tailles heterogenes, le separateur utilise est
                  la tabulation*/
                st = new StringTokenizer(entree.readLine(), "\t");
                for (int i = 0; i < model.nb_pop; i++) {
                    model.N[i] = Integer.valueOf(st.nextToken()).intValue();
                }
            }

            // Autres cas
            if ((!model.rep_tailles.equals("H")) &&
                (!model.rep_tailles.equals("NH"))) {
                System.out.println("Error of file data entry:" +
                                   model.rep_tailles);
                System.exit(1);
            }

            model.nbcop = new float[model.nb_pop];
            model.iles = new Population[model.nb_pop];
            st = new StringTokenizer(entree.readLine(), "\t");

            /* saisie du nombre
                         moyen de copies par population*/
            for (int i = 0; i < model.nb_pop; i++) {
                model.nbcop[i] = Float.valueOf(st.nextToken()).floatValue();
            }

            // Traitement du cas d'une explosion locale
            if (model.conta_mod.equals("LB")) {
                InitIslanModelFromFile.initLB(model, entree);
            }
            // Traitement du cas d'une copie agressive
            else if (model.conta_mod.equals("AC1") ||
                     model.conta_mod.equals("AC2")) {
                InitIslanModelFromFile.initAC1(model, entree);
            }

            // Initialisation des iles
            model.setPopulations();

            model.imigration = new int[model.nb_pop][model.nb_pop - 1];
            model.voisinage = new Population[model.nb_pop][model.nb_pop - 1];
            model.tx_migration = new float[model.nb_pop][model.nb_pop];
            for (int i = 0; i < model.nb_pop; i++) {
                model.tx_migration[i][i] = (float) model.N[i];
            }
            model.rep_tx = entree.readLine();

            // Cas d'une diffusion homogene
            if (model.rep_tx.equals("H")) {
                model.tx_migration[1][0] =
                        Float.valueOf(entree.readLine()).floatValue();
                for (int i = 0; i < model.nb_pop; i++) {
                    for (int j = 0; j < i; j++) {
                        model.tx_migration[i][j] = model.tx_migration[1][0];
                        model.tx_migration[j][i] = model.tx_migration[i][j];
                        model.tx_migration[i][i] -= model.tx_migration[i][j];
                        model.tx_migration[j][j] -= model.tx_migration[j][i];
                    }
                }
            }

            // Cas d'une diffusion heterogene
            if (model.rep_tx.equals("NH")) {
                for (int i = 1; i < model.nb_pop; i++) {
                    st = new StringTokenizer(entree.readLine(), "\t");
                    for (int j = 0; j < i; j++) {
                        model.tx_migration[i][j] = Float.valueOf(
                                st.nextToken()).floatValue();
                        model.tx_migration[j][i] = model.tx_migration[i][j];
                        model.tx_migration[i][i] -= model.tx_migration[i][j];
                        model.tx_migration[j][j] -= model.tx_migration[j][i];
                    }
                }
            }

            // Autres cas
            if ((!model.rep_tx.equals("H")) && (!model.rep_tx.equals("NH"))) {
                System.out.println("Erreur");
                System.exit(1);
            }

            entree.close();

        }

        catch (IOException e) {
            e.printStackTrace();
        }

        model.setNeightbourhood();

        return;
    }


    /**
     * initLB
     * A specific method that assign the parameters of LB models
     * @param model IslandModel
     * @param entree BufferefReader
     */
    public static void initLB(IslandModel model, BufferedReader entree)
    {

        try {
            // rcupration des points chauds
            StringTokenizer st = new StringTokenizer(entree.readLine(), "\t");
            model.hot_spots = new boolean[model.nb_pop];
            for (int i = 0; i < model.nb_pop; i++) {
                model.hot_spots[i] = (new Boolean(st.nextToken())).booleanValue();
            }

            // Initialisation des taux de transposition
            st = new StringTokenizer(entree.readLine(), "\t");
            transElement.setTransRates(
                    Float.valueOf(st.nextToken()).floatValue(),
                    Float.valueOf(st.nextToken()).floatValue());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * A specific method that assign the parameters of AC1 models
     * @param model IslandModel
     * @param entree BufferedReader
     */
    public static void initAC1(IslandModel model, BufferedReader entree)
    {
        try {

            /* saisie du nombre
                             moyen de copies agressives par population*/
            StringTokenizer st = new StringTokenizer(entree.readLine(),
                    "\t");
            model.nbcop2 = new float[model.nb_pop];
            for (int i = 0; i < model.nb_pop; i++) {
                model.nbcop2[i] = Float.valueOf(st.nextToken()).floatValue();
            }

            // Initialisation des taux de transposition
            st = new StringTokenizer(entree.readLine(), "\t");
            transElement.setTransRates(
                    Float.valueOf(st.nextToken()).floatValue(),
                    Float.valueOf(st.nextToken()).floatValue());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
