package projet_these.et;

import java.util.*;

/**
 * Implementation of a genome with two types of copies
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 20/12/02
 */
public class IndividuPoly1 extends Individu
{
    
    /**
     * Number of invasives copies of the individual
     */
    protected int nb_copies2;
    
    /**
     * Types of the chromosomes pairs
     */
    protected static final ChromoPairPoly chromo_pairs_class
            = new ChromoPairPoly();
    
    
    /**
     * Empty Contructor of an individual
     */
    public IndividuPoly1()
    {
    }
    
    
    /**
     * Contructor of initialization of the individuals
     * @param   n1  Total number of copies of the genome
     * @param   n2  Number of agressive copies
     */
    public IndividuPoly1(int n1, int n2)
    {
        // Inserts number on each pairs
        int[][] ni = new int[2][3];
        
        // Genome copies number initiation
        nb_copies = n1;
        nb_copies2 = n2;
        
        // Running of positionning process
        transposition(nb_copies, ni[0], 1f);
        transposition(nb_copies2, ni[1], 1f);
        
        // Chromosomes pair initiation
        XX = new ChromoPairPoly[3];
        for (int i = 0; i < 3; i++)
            XX[i] = new ChromoPairPoly(ni[0][i], ni[1][i]);
        
        // Transposition state initiation
        tran_exc = false;
    }
    
    
    /**
     * Procedure setting a genome
     * @param X First pair of chromosomes
     * @param Y Second pair of chromosomes
     * @param Z Third pair of chromosomes
     * @return Individu the genome setted
     */
    protected Individu setChromoPairs(ChromosomesPair X, ChromosomesPair Y, ChromosomesPair Z)
    {
        // 3 pairs of chromosoms in XX
        XX = new ChromoPairPoly[3];
        XX[0] = X;
        XX[1] = Y;
        XX[2] = Z;
        
        // Genomic copies number initiation
        nb_copies = XX[0].nb_ins + XX[1].nb_ins + XX[2].nb_ins;
        nb_copies2 = ((ChromoPairPoly) XX[0]).nb_agr +
                ((ChromoPairPoly) XX[1]).nb_agr +
                ((ChromoPairPoly) XX[2]).nb_agr;
        
        // Transposition state initiation
        tran_exc = false;
        
        return this;
    }
    
    
    /**
     * Function of segregation of the homologous pairs of chromosomes in meiose
     * @return the table of the separate chromosomes of their counterpart,
     * i.e. constitutive of the gamete
     */
    protected Hashtable[] gametes()
    {
        
        // Initialisation of the haplotype
        Hashtable[] haplo = new Hashtable[3];
        
        // Transposition/Excision of all germinales cells in the same way
        if (! tran_exc)
        {
            int[] nb_x = new int[3];
            transposition(this.nb_copies, nb_x,
                    transElement.getNormTransRate());
            insCop(nb_x, false);
            transposition(this.nb_copies2, nb_x,
                    transElement.getInvaTransRate());
            insCop(nb_x, true);
            delCop();
            
            // Transposition state changing
            tran_exc = true;
        }
        
        // Homologues separation during meiotic step
        for (int i = 0; i < 3; i++)
            haplo[i] = XX[i].segregation();
        
        return haplo;
    }
    
    
    /**
     * Functin returning selected entity
     * @return  Number of selected elements
     */
    protected int getEntite_sel()
    {
        return (this.nb_copies + this.nb_copies2);
    }
}