package projet_these.et;

import java.io.Serializable;;

/**
 * Implementation of a transposable element (TE)
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 18/12/02
 */
public class transElement extends geneAllele implements Serializable, Cloneable
{

    /**
     * Default transposition rate
     */
    public final static float DEFAULT_TRANSPOSITION_RATE = 2.E-3f;

    /**
     * Default excision rate
     */
    public final static float DEFAULT_EXCISION_RATE = 5.E-5f;

    /**
     * Normal transposition rate
     */
    private static float norm_trans_rate = DEFAULT_TRANSPOSITION_RATE;

    /**
     * Agressive transposition rate
     */
    private static float inva_trans_rate = DEFAULT_TRANSPOSITION_RATE;

    /**
     * Excision rate
     */
    private static float exci_rate = DEFAULT_EXCISION_RATE;


    /**
     * Default contructor of initialization of the transposable
     * elements. It is regarded as being normal
     */
    public transElement()
    {
        this(false);
    }


    /**
     * Contructor of initialization of the transposble element
     * @param agressive Boolean  The type of TE (normal or agressive)
     */
    public transElement(boolean agressive)
    {
        super(agressive);
    }


    /**
     * Procedure setting identicals transpositions rates
     * @param   rate    transposition rate of the two types copies
     */
    public static void setTransRates(float rate)
    {
        transElement.setTransRates(rate, rate);
    }


    /**
     * Procedure setting differents transpositions rates
     * @param norm_r The transposition rate of normale copies
     * @param inva_r The transposition rate of agressive copies
     */
    public static void setTransRates(float norm_r, float inva_r)
    {
        transElement.norm_trans_rate = norm_r;
        transElement.inva_trans_rate = inva_r;
    }


    /**
     * Procedure setting excisions rate
     * @param rate The excision rate
     */
    public static void setExciRate(float rate)
    {
        transElement.exci_rate = rate;
    }


    /**
     * Procedure returning normale transposition rate
     * @return  The normal transposition rate
     */
    public static float getNormTransRate()
    {
        return transElement.norm_trans_rate;
    }


    /**
     * Procedure returning agressive transposition rate
     * @return  The invasive copies tranposition rate
     */
    public static float getInvaTransRate()
    {
        return transElement.inva_trans_rate;
    }


    /**
     * Procedure returning excision rate
     * @return  The exision rate
     */
    public static float getExciRate()
    {
        return transElement.exci_rate;
    }


    /**
     * Creates an agressive clone
     * @return Object
     */
    public Object clone()
    {
        return new transElement(this.agressive);
    }
}