/*************************************************************************
 *         Copyright notice for JFreeChart and JCommon libraries          *
 *                                                                        *
 *The present application links to JFreeChart and JCommon libraries as    *
 *external jar files that are under the GNU Lesser General Public License.*
 *You can find a copy of that license at the root folder containing       *
 *the main class of TESD. More informations such as complete source codes *
 *to modify the libraries are available at this web site: www.jfree.org.  *
 *************************************************************************/


/*
 * Graphics.java
 *
 * Created on 3 fvrier 2005, 12:43
 */

package projet_these.ig;

import javax.swing.*;
import java.util.*;
import java.io.*;


/**
 * Graphiques is a general class extended by Curves, Histograms or Maps to give them
 * default functions as a JFileChooser, a table[][] filled from the choosed "simul" file,
 * and setted it.
 * @author Yann Letrillard
 */
public class Graphiques extends JInternalFrame {
    /**
     * Instance of the MainFrame JFrame container of Graphics JInternalFrame
     */
    private MainFrame super_frame;
    /**
     * Defines the file containing output data from simulation for naming
     * the graphic and read it
     */
    protected String output_file_name;
    /**
     * Defines the type of graphics: curves, maps, histograms
     */
    protected String graphiques_type;
    
    /**
     * The matrix containing all data from "simul" data file
     */
    protected float [][]output_table;
    
    /**
     * The number of lines in the data table
     */
    protected int table_heigth;
    
    /**
     * The number of columns in the data table
     */
    protected int table_width;
    
    /**
     * The max value of table
     */
    private float table_max;
    
    /**
     * A boolean determining if the Graphiques have to be visible or not
     */
    public boolean dispose = true;
    private String empty_warning;
    
    /**
     * Creates a new instance of Graphics by calling the chooseYourFile method,
     *  puts the data of the output choosen file in a bi-dimentional float table,
     *  constructs a JInternalFrame and set it up (background, size, position...),
     *  & add the JPanel that paints the specified graphics in graphic_class_name
     * @param output_file_name The file containing data from a simulation that
     * will be ploted
     * @param super_frame MainFrame an instance of the JFrame container
     * @param graphic_class_name String name of the specific graphic to represent;
     *  used as meta-data
     * @param default_construction boolean if true, setDataFile and setOutputTable methods are automatically called.
     * If false, the inheritant Gaphiques class can must call.
     */
    public Graphiques(MainFrame super_frame, String graphic_class_name, boolean default_construction, String output_file_name) {
        
        //Constructor of a JInternalFrame resizable, closable, maximizable, iconifiable.
        super(graphic_class_name, true, true, true, true);
        
        //set the parent frame
        this.setSuperFrame(super_frame);
        
        //set the kind of grahics
        this.setGraphicsType(graphic_class_name);
        
        if (graphic_class_name == "Histogram")
            setEmpty_warning("The number of generations chosen is too small \n" +
                "for histogram to be reliably determined: \n" +
                "this number needs to be over 10 000");
        else 
            this.setEmpty_warning("The file is empty! \nBad simulation settings?");
        //
        if (default_construction && output_file_name == ""){
            this.setOutputFile();
            this.setOutputTable();
        } else if (default_construction && output_file_name != ""){
            this.output_file_name = output_file_name;
            this.setOutputTable();
        }
        
        else if (!default_construction && output_file_name != ""){
            this.output_file_name = output_file_name;
        }
        
        setupGraphiquesFrame();
        
    }
    
    /**
     * Construct a JFileChooser to set the file containing the data to represent
     */
    protected void setOutputFile() {
        
        //The local string contaning the path
        String file = "";
        
        //chooser is the instance of this JFileChooser
        JFileChooser chooser = new JFileChooser();
        
        ExampleFileFilter filter = new ExampleFileFilter();
        
        if (this.graphiques_type == "Curves" || this.graphiques_type == "Maps"){
            filter.addExtension("mea");
            filter.addExtension("var");
            filter.setDescription("TESD mean or variance files");
        }
        if (this.graphiques_type == "Histogram") {
            filter.addExtension("frq");
            filter.setDescription("TESD frequencies files");
        }
        
        chooser.setFileFilter(filter);
        
        //add an exception event
        if(super_frame.getCurrentPath() != null)
            chooser.setCurrentDirectory(new File(super_frame.getCurrentPath()));
        
        //a value (determining if a file is choosen) is returned as retVal
        int return_value = chooser.showDialog(super_frame, null);
        
        if (return_value == chooser.APPROVE_OPTION) {
            //setting of the global path to be portable on different systems
            file = chooser.getSelectedFile().getAbsolutePath();
            super_frame.setCurrentPath(file.substring(0,
                    file.lastIndexOf(System.getProperty("file.separator"))));
            if (file.indexOf("simul") == -1) {
                String error = "The file doesn't contain the 'simul' prefix!" +
                        "\nIt must be done by a TESD simulation";
                JOptionPane.showMessageDialog(getSuperFrame(), error,
                        "", JOptionPane.ERROR_MESSAGE);
                this.disposeGraphic(true);
            } else {
                this.disposeGraphic(false);
                this.output_file_name = file;
            }
            
            
        } else {
            this.disposeGraphic(true);
        }
    }
    
    
    
    /**
     * Fill table from the choosen "simul" file
     */
    protected void setOutputTable() {
        
        //the buffer to open a stream from the file
        BufferedReader entree = null;
        
        //the string tokenizer to get tokens from line
        StringTokenizer st;
        
        //line readed from the file
        String line;
        
        //value readed from the file by the tokenizer
        String value;
        
        try {
            // Initialisation of a reading buffer from a text file
            entree = new BufferedReader(new FileReader(this.output_file_name));
            
            // Calculation of the width and heght of the matrix
            FileReader fr = new FileReader(output_file_name);
            
            //number of tabulations and number of lines
            int c, nb_tab = 0, nb_cr = 0;
            while ((c = fr.read()) != -1) {
                nb_tab += (c == '\t') ? 1 : 0;
                nb_cr += (c == '\n') ? 1 : 0;
            }
            
            //if the data file is void, a popup signals it and the graphic will not open
            if (nb_cr <= 2){
                emptyFileWarning();
            } else {
                this.disposeGraphic(false);
            }
            
            //setting the height and width of the table
            this.table_heigth = nb_cr;
            this.table_width = (nb_tab / nb_cr)+1;
            
            // Creation of the data table
            output_table = new float[this.table_heigth][this.table_width];
            
            
            // First line uninteresting as containing number of populations
            entree.readLine();
            
            // Fill the table for the file's data
            for (int i = 0; i < this.table_heigth; i++) {
                
                // A ith line is put in String line
                line = entree.readLine();
                
                // Delimiter is one tabulaion
                st = new StringTokenizer(line, "\t");
                
                for (int j = 0; j < this.table_width; j++) {
                    value = st.nextToken();
                    output_table[i][j] = Float.valueOf(value.replace(',', '.')
                    ).floatValue();
                    //get max from table without counting the first column
                    table_max = (output_table[i][j] > table_max & j != 0) ? output_table[i][j] : table_max;
                }
            }
            
            //close the stream for the system
            entree.close();
            
        } catch (Exception e) {
        }
    }
    
    private void emptyFileWarning() {
        JOptionPane.showMessageDialog(getSuperFrame(), empty_warning,
                "", JOptionPane.WARNING_MESSAGE);
        this.disposeGraphic(true);
    }
    
    /**
     * Set size default close operation and location of the Graphiques
     */
    private void setupGraphiquesFrame() {
        
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        this.setSize(500, 300);
        //further Graphics locates lower in MainFrame
        setLocation(super_frame.getComponentCount() * 10 %
                (super_frame.getSize().width / 2),
                super_frame.getComponentCount() *
                10 % (super_frame.getSize().height / 2));
    }
    
    /**
     * getter returning the name of the "simul" file choosen
     * @return String file the path of the "simul" file
     */
    public String getFileName() {
        return this.output_file_name;
    }
    
    /**
     * Procedure returning a reference to main frame
     * @return MainFrame a reference to main frame
     */
    public MainFrame getSuperFrame() {
        return this.super_frame;
    }
    
    /**
     * Set the super_frame field as an intance of the main frame from costructor arguments
     * @param super_frame MainFrame instance argument in the constructor
     */
    public void setSuperFrame(MainFrame super_frame) {
        this.super_frame = super_frame;
    }
    
    /**
     * return the number of lines in the table
     * @return int the number of lines in the matrix table
     */
    public int getTableHeight() {
        return this.table_heigth;
    }
    
    /**
     * Returns the number of columns in the matrix table
     * @return int the number of the columns in the matrix table
     */
    public int getTableWidth() {
        return this.table_width;
    }
    
    /**
     * Return the max value from table. Setted by setOutputTable().
     * @return float The maximum value from table
     */
    public float getTableMax() {
        return this.table_max;
    }
    
    /**
     * Used by MapSettings. Allow the non apearance of the Graphiques if needed.
     * @param isTrue Boolean deciding if the curent Graphique have to be disposed.
     */
    public void disposeGraphic(boolean isTrue){
        this.dispose = isTrue;
    }
    
    /**
     * Set the type of the Graphiques
     * @param class_type String define the name of the curent Graphiques object ("Maps", "Curves", "Histograms")
     */
    private void setGraphicsType(String class_type){
        this.graphiques_type = class_type;
    }

    public String getEmpty_warning() {
        return empty_warning;
    }

    public void setEmpty_warning(String empty_warning) {
        this.empty_warning = empty_warning;
    }
}
