/**************************************************************************
 *                Copyright notice for JavaHelp library                    *
 *                                                                         *
 *The present application links to JavaHelp library as an external jar file*
 * that is under the Sun Microsystems, Inc. Binary Code License Agreement and*
 *supplemental license term fot JavaHelp version 2.0.                      *
 *You can find a copy of that license at the root folder containing the    *
 *main class of TESD. More informations are available at this web site:    *
 *http://java.sun.com/products/javahelp                                    *
 **************************************************************************/

/*************************************************************************
 *         Copyright notice for JFreeChart and JCommon libraries          *
 *                                                                        *
 *The present application links to JFreeChart and JCommon libraries as    *
 *external jar files that are under the GNU Lesser General Public License.*
 *You can find a copy of that license at the root folder containing       *
 *the main class of TESD. More informations such as complete source codes *
 *to modify the libraries are available at this web site: www.jfree.org.  *
 *************************************************************************/

package projet_these.ig;

import javax.swing.border.BevelBorder;
import javax.swing.border.EtchedBorder;
import projet_these.es.InitIslanModelFromFile;
import projet_these.et.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.net.URL;
import java.io.*;
import java.io.File;
import javax.help.*;
import projet_these.es.OutPutDataFiles;




/**
 * Implementation of the main software frame
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 01/03/02
 */
public class MainFrame extends JFrame {
    
    /**
     * Container for JInternalFrames
     */
    protected JDesktopPane desktop;
    
    /**
     * Simulation menu
     */
    protected JMenu menu;
    
    /**
     * Graphics menu
     */
    protected JMenu menu2;
    
    /**
     * Help menu
     */
    protected JMenu menu3;
    
    /**
     * Menu Simulation item Online
     */
    protected JMenuItem item1;
    
    /**
     * Menu Simulation item InputFile
     */
    protected JMenuItem item2;
    
    /**
     * Menu Simulation item Run
     */
    protected JMenuItem item3;
    
    /**
     * Menu Simulation item Quit
     */
    protected JMenuItem item4;
    
    /**
     * Menu Graphics item Curves
     */
    protected JMenuItem item11;
    
    /**
     * Menu Graphics item Histograms
     */
    protected JMenuItem item12;
    
    /**
     * Menu Graphics item Maps
     */
    protected JMenuItem item13;
    
    /**
     * Menu Help item help content
     */
    protected JMenuItem item21;
    
    /**
     * Menu Help item About
     */
    protected JMenuItem item22;
    
    /**
     * Menu item guide at the right
     */
    protected JMenuItem itemb4;
    
    /**
     * Menu bar of MainFrame moi
     */
    private JMenuBar mb;
    
    /**
     * The label that display the current dir path bottom at the page
     */
    private JLabel cur_dir;
    
    /**
     * This current main frame of TESD
     */
    private MainFrame moi;
    
    /**
     * The current Island Model
     */
    protected IslandModel model;
    
    /**
     * Information panel on simulation entries
     */
    protected Entries entries;
    
    /**
     * Current working directory
     */
    private String current_path;
    
    /**
     * Height of the main frame
     */
    protected int height;
    
    /**
     * Width of the main frame
     */
    protected int width;
    
    /**
     * The welcoming picture at the begining of the program
     */
    protected JWindow window;
    
    /**
     * Tels if the main frame as to be visible (command line mode)
     */
    public boolean is_visible;
    
    
    /**
     * Contructor of initialization of the frame
     * @param s String representing software title
     * @param m Reference to the current model
     * @param show Boolean for set visible; true in case
     * of GUI mode and false for advanced user mode
     */
    public MainFrame(String s, boolean show, IslandModel m) {
        
        // Utilisation du constructeur de JFrame
        super(s);
        
        // Recuperation de la reference sur le modele courant
        model = m;
        
        is_visible = show;
        
        // Reglage de l'apparence des fenetres
        try {
            UIManager.setLookAndFeel(UIManager.
                    getCrossPlatformLookAndFeelClassName());
            SwingUtilities.updateComponentTreeUI(this);
        } catch (UnsupportedLookAndFeelException e) {
            System.out.println("This Look & Feel is not " +
                    "available on your operating system.");
        } catch (IllegalAccessException e) {
            System.out.println("This Look & Feel is not " +
                    "available on your operating system.");
        } catch (ClassNotFoundException e) {
            System.out.println("This Look & Feel was not found.");
        } catch (InstantiationException e) {
            System.out.println("This Look & Feel cannot " +
                    "be instanciated.");
        } catch (Exception e) {
            System.out.println("Running error.");
        }
        
        // Gestion de la fermeture de la fenetre
        addWindowListener(new MaFenetre_adapter());
        
        // Positionnement et dimensionement de la fenetre
        setup();
        
        /* Recuperation de l'instance courante afin de pouvoir la traiter
        dans des classes internes de gestion des evenements*/
        moi = this;
        
        // Creation des menus principaux
        menu = new JMenu("Simulations", true);
        menu.setToolTipText("To define the simulation parameters and run them");
        menu2 = new JMenu("Graphics", true);
        menu2.setToolTipText("To see the results of simulations");
        menu2.setEnabled(true);
        menu3 = new JMenu("Help", true);
        menu3.setToolTipText("A quick help for a first experience of TESD");
        
        
        // Creation des items du menu
        buildItem1();
        buildItem2();
        menu.add(new JSeparator());
        buildItem3();
        menu.add(new JSeparator());
        buildItem4();
        
        // Creation des items du menu2
        buildItem11();
        buildItem12();
        buildItem13();
        buildItem14();
        
        // Creation des items du menu3
        buildItem21();
        buildItem22();
        
        // Creation d'une barre de menus
        mb = new JMenuBar();
        
        // Ajout des menus  mb
        mb.add(menu);
        mb.add(menu2);
        mb.add(menu3);
        buildMenubarItem4();
        
        //mb.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        
        /* Modification de la barre de menu utilisee par
        la fenetre courante*/
        setJMenuBar(mb);
        
        // Creation du conteneur de gestion des fenetres internes
        desktop = new JDesktopPane();
        desktop.setBackground(new Color(160, 160, 160));
        desktop.setBorder(BorderFactory.createLineBorder(Color.BLACK, 2));// EtchedBorder(EtchedBorder.LOWERED));
        getContentPane().add(desktop, BorderLayout.CENTER);
        
        // Permet de rendre la fenetre courante visible
        //setVisible(show);
        
        // Initialisation du chemin courant
        try { if(this.getCurrentPath() == null
                || this.getCurrentPath().equals("")) {
            this.setCurrentPath((new File(".")).getCanonicalPath());
        }} catch(Exception e) {};
        
        
        JPanel state_bar = new JPanel();
        state_bar.setLayout(new GridLayout());
        cur_dir = new JLabel("Current directory : " + getCurrentPath());
        cur_dir.setFont(new Font("Dialog", Font.PLAIN, 12));
        state_bar.add(cur_dir);
        state_bar.setVisible(true);
        //state_bar.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
        getContentPane().add(state_bar,BorderLayout.SOUTH);
        
        
        
        if (is_visible){
            showWelcome();
            //moi.setVisible(is_visible);
            //    showEntrance();
        }
        
    }
    
    private void showWelcome(){
        window = new JWindow(moi);
        //JInternalFrame window = new JInternalFrame();
        ImageIcon icone = null;
        //String separator = System.getProperty(File.pathSeparator);
        //String icone_path = "images" + java.lang.System.getProperty("file.separator") + "welcome.png";
        String icone_path = "images/welcome.png";
        //String icone_path = "images" + File.separator + "welcome.png";
        URL image_url = MainFrame.class.getResource(icone_path);
        icone = new ImageIcon(image_url);
        
        JLabel image = new JLabel(icone);
        window.getContentPane().add(image);
        int x = (int) moi.getLocation().x + (moi.getWidth()/2);
        int y = (int) moi.getLocation().y + (moi.getHeight()/2);
        int x2 = (int) x - (image.getIcon().getIconWidth()/2);
        int y2 = (int) y - (image.getIcon().getIconHeight()/2);
        window.setSize(icone.getIconWidth(), icone.getIconHeight());
        window.setLocation(x2, y2);
        window.setVisible(true);
        int delay = 2000; //milliseconds
        ActionListener taskPerformer = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                window.dispose();
                moi.setVisible(is_visible);
                showEntrance();
            }
        };
        Timer timer = new Timer(delay, taskPerformer);
        timer.setRepeats(false);
        timer.start();
    }
    
    
    
    /**
     * Setup procedure of MainFrame
     */
    private void setup() {
        
        // Dimensions de la fenetre
        this.width = 650;
        this.height = 490;
        setSize(width, height);
        
        // Dimensions de l'ecran
        Dimension tailleEcran = Toolkit.getDefaultToolkit().getScreenSize();
        int largeurEcran = tailleEcran.width;
        int hauteurEcran = tailleEcran.height;
        
        // Positionnement de la fenetre
        int xPos = (largeurEcran - width) / 2;
        int yPos = (hauteurEcran - height) / 2;
        setLocation(xPos, yPos);
    }
    
    private void showEntrance(){
        Entrance welcome = new Entrance(this,true,this);
        
        int welcomeX = (int)this.getBounds().getCenterX()-(welcome.getWidth()/2);
        int welcomeY = (int)this.getBounds().getCenterY()-(welcome.getHeight()/2);
        
        welcome.setLocation(new Point(welcomeX, welcomeY));
        welcome.setVisible(true);
    }
    
    /**
     *Construction of the menu item "Guide" that launch the forms helping users
     */
    private void buildMenubarItem4(){
        itemb4 = new JMenuItem("Guide");
        itemb4.setToolTipText("If you're lost click here!");
        itemb4.setMaximumSize(new Dimension(0, 50));
        itemb4.setBackground(Color.white);
        itemb4.setForeground(Color.red);
        itemb4.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        mb.add(Box.createHorizontalGlue());
        itemb4.addActionListener(new guide_AL());
        mb.add(itemb4);
    }
    
    
    /**
     * Construction procedure of Item On Line of Simulations
     */
    private void buildItem1() {
        
        // Instanciation d'un item On Line
        item1 = new JMenuItem("New simulation");
        
        // Classe interne de gestion des evenements de l'item courant
        item1.addActionListener(new onLine_AL());
        
        // Ajout de l'item1 au menu
        menu.add(item1);
    }
    
    
    /**
     * Construction procedure of Item Input File of Simulations
     */
    private void buildItem2() {
        
        // Instanciation d'un item Input File
        item2 = new JMenuItem("Open .para file");
        item2.setToolTipText("To open a file with previously defined simulation");
        
        // Classe interne de gestion des evenements de l'item courant
        item2.addActionListener(new inputFile_AL());
        
        // Ajout de l'item2 au menu
        menu.add(item2);
    }
    
    
    /**
     * Construction procedure of Item Run of Simulations
     */
    private void buildItem3() {
        
        // Instanciation d'un item Run
        item3 = new JMenuItem("Run");
        
        // Classe interne de gestion des evenements de l'item courant
        item3.addActionListener(new run_AL());
        
        // Ajout de l'item3 au menu
        menu.add(item3);
        
        /* L'item ne doit pas etre actif sans instance
        completement specifiee du model courant*/
        item3.setEnabled(false);
    }
    
    
    /**
     * Construction procedure of Item Quitter of Simulations
     */
    private void buildItem4() {
        
        // Instanciation d'un item Quitter
        item4 = new JMenuItem("Quit");
        item4.setToolTipText("Close TESD?");
        
        // Classe interne de gestion des evenements de l'item courant
        item4.addActionListener(new quit_AL());
        
        // Ajout de l'item4 au menu
        menu.add(item4);
    }
    
    
    /**
     * Construction procedure of Item Curves of Graphics
     */
    private void buildItem11() {
        
        // Instanciation d'un item Curves
        JMenuItem item11 = new JMenuItem("Curves");
        item11.setToolTipText("The values of means or variances of copy numbers through generations");
        // Classe interne de gestion des evenements de l'item courant
        item11.addActionListener(new curves_AL());
        
        // Ajout de l'item11 au menu2
        menu2.add(item11);
    }
    
    
    /**
     * Construction procedure of Item Histogram of Graphics
     */
    private void buildItem12() {
        
        // Instanciation d'un item Histogram
        JMenuItem item12 = new JMenuItem("Histograms");
        item12.setToolTipText("Distribution of the frequencies of insertions");
        
        // Classe interne de gestion des evenements de l'item courant
        item12.addActionListener(new histo_AL());
        
        // Ajout de l'item12 au menu2
        menu2.add(item12);
    }
    
    
    /**
     * Construction procedure of Item Maps of Graphics
     */
    private void buildItem13() {
        
        // Instanciation d'un item Maps
        JMenuItem item13 = new JMenuItem("Maps");
        item13.setToolTipText("Populations as bubbles of size proportionnal to their TE copy number");
        
        // Classe interne de gestion des evenements de l'item courant
        item13.addActionListener(new maps_AL());
        
        // Ajout de l'item13 au menu2
        menu2.add(item13);
    }
    
    /**
     * Construction procedure of Item Maps of Graphics
     */
    private void buildItem14() {
        
        // Instanciation d'un item Animated Map
        JMenuItem item14 = new JMenuItem("Animated Map");
        item14.setToolTipText("Like 'Maps' but in a 'movie player' way");
        
        // Classe interne de gestion des evenements de l'item courant
        item14.addActionListener(new mapsanim_AL());
        
        // Ajout de l'item14 au menu2
        menu2.add(item14);
    }
    
    /**
     * Construction procedure of Item Contents of Help
     */
    private void buildItem21() {
        
        // Instanciation d'un item Contents
        JMenuItem item21 = new JMenuItem("Contents");
        
        // Classe interne de gestion des evenements de l'item courant
        item21.addActionListener(new contents_AL());
        
        // Ajout de l'item21 au menu3
        menu3.add(item21);
    }
    
    
    /**
     * Construction procedure of Item About of Help
     */
    private void buildItem22() {
        
        // Instanciation d'un item About
        JMenuItem item22 = new JMenuItem("About");
        
        // Classe interne de gestion des evenements de l'item courant
        item22.addActionListener(new about_AL());
        
        // Ajout de l'item22 au menu3
        menu3.add(item22);
    }
    
    
    /**
     * Procedure setting the current directory
     * @param   cp  The current working directory
     */
    protected void setCurrentPath(String cp) {
        
        this.current_path = cp;
        cur_dir.setText("Current Directory: " + getCurrentPath());
    }
    
    
    /**
     * Procedure sending the current directory
     * @return   The current working directory
     */
    protected String getCurrentPath() {
        
        return this.current_path;
    }
    
    
    /**
     * Procedure displaying the set of current parameters
     */
    protected void setEntries() {
        
        try {
            if (entries != null){
                entries.setClosed(true);
            }
        } catch (java.beans.PropertyVetoException p_e) {
            /*Imprime sur la sortie standard ou sur un stream,
              l'exception et la trace de l'exception dans la pile*/
            p_e.printStackTrace();
        }
        entries = new Entries(this);
        desktop.add(entries);
        try {
            entries.setSelected(true);
        } catch (java.beans.PropertyVetoException ve) {
        };
    }
    
    
    
    /**
     * Implementation of the main software frame adapter
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class MaFenetre_adapter extends WindowAdapter {
        
        /**
         * Procedure closing the sofware frame
         * @param  e   a click on the top right box
         */
        public void windowClosing(WindowEvent e) {
            
            /* Si une copie disque temporaire du modele existe,
            elle sera supprimee  la fermeture de la fenetre*/
            try {
                if (model.modelFile != null){
                    model.modelFile.delete();
                }
            }
            
            // Renvoie le message detaille associe  l'exception
            catch (SecurityException sec_exc) {
                sec_exc.getMessage();
            }
        }
    }
    
    
    /**
     * Implementation of guide item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class guide_AL implements ActionListener {
        
        /**
         * Procedure opening the Entrance form
         * @param  e   a click on the Guide mznu
         */
        public void actionPerformed(ActionEvent e) {
            showEntrance();
        }
    }
    
    /**
     * Implementation of On Line item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class onLine_AL implements ActionListener {
        
        
        /**
         * Procedure opening the On Line Internal Frame
         * @param  e   a click on the On Line item
         */
        public void actionPerformed(ActionEvent e) {
            
            // Si un modele existait dej, il est supprime
            try {
                if (model.modelFile != null){
                    model.modelFile.delete();
                }
            } catch (SecurityException sec_exc) {
                /*Imprime sur la sortie standard ou sur un stream,
                 l'exception et la trace de l'exception dans la pile*/
                sec_exc.printStackTrace();
            }
            
            /* Creation d'une fenetre interne de saisie en ligne
             des parametres*/
            OnLine on_line = new OnLine(moi);
            desktop.add(on_line);
            try {
                on_line.setSelected(true);
            } catch (java.beans.PropertyVetoException ve) {
            }
            
            // Desactivation des menus Simulations
            menu.setEnabled(false);
            
            /* Desactivation de l'item Run et du menu des
            representations graphiques*/
            item3.setEnabled(false);
            
            // Initialisation des compteurs de Simulation
            Simulation.initMeters();
        }
    }
    
    
    /**
     * Implementation of Input  File item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class inputFile_AL implements ActionListener {
        
        
        /**
         * Procedure opening the Input File Internal Frame
         * @param  e   a click on the Input File item
         */
        public void actionPerformed(ActionEvent e) {
            
            /*try {
                if (model.modelFile != null){
                    model.modelFile.delete();
                }
            } catch (SecurityException sec_exc) {
                /*Imprime sur la sortie standard ou sur un stream,
                 l'exception et la trace de l'exception dans la pile*
                sec_exc.printStackTrace();
            }
             
            /* Instanciation de la fenetre interne d'initialisation
              partir d'un fichier*
            FileDataEntry fde = new FileDataEntry(moi);
            desktop.add(fde);
            try {
                fde.setSelected(true);
            } catch (java.beans.PropertyVetoException ve) {
            }*/
            
            
            /**
             * Let the user choosing an input file to see its graph
             */
            
            
            //The local string contaning the path
            String file = "";
            
            //chooser is the instance of this JFileChooser
            JFileChooser chooser = new JFileChooser();
            
            ExampleFileFilter filter = new ExampleFileFilter();
            
            filter.addExtension("para");
            filter.setDescription("Simulation's parameters file");
            
            chooser.setFileFilter(filter);
            
            //make the file chooser to open in the current directoy of MainFrame
            if(moi.getCurrentPath() != null)
                chooser.setCurrentDirectory(new File(moi.getCurrentPath()));
            
            //a value (determining if a file is choosen) is returned as retVal
            int return_value = chooser.showDialog(moi, null);
            
            if (return_value == chooser.APPROVE_OPTION) {
                //setting of the global path to be portable on different systems
                file = chooser.getSelectedFile().getAbsolutePath();
                moi.setCurrentPath(file.substring(0,
                        file.lastIndexOf(System.getProperty("file.separator"))));
                
                String simul_name = file.toString().substring(0, file.indexOf(".para", 0));
                
                moi.model.nomFic1 = file.toString();
                moi.model.nomFic2 = simul_name;
                InitIslanModelFromFile.initModel(moi.model);
                moi.setEntries();
                moi.item3.setEnabled(true);
                moi.menu.setEnabled(true);
                SimulationForm next_form = new SimulationForm(moi, true, moi);
                OutPutDataFiles.checkExistingOutputFiles(new File(file));
                // Initialisation des compteurs de Simulation
                Simulation.initMeters();
            }
        }
    }
    
    /**
     * Implementation of Run item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class run_AL implements ActionListener {
        
        
        /**
         * Procedure opening the Run Internal Frame
         * @param  e   a click on the Run item
         */
        public void actionPerformed(ActionEvent e) {
            
            // Instantiation d'une Simulation
            Simulation uneSimule = new Simulation(moi);
            desktop.add(uneSimule);
            try {
                uneSimule.setSelected(true);
            } catch (java.beans.PropertyVetoException ve) {
            }
        }
    }
    
    
    /**
     * Implementation of Quit item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class quit_AL implements ActionListener {
        /**
         * Procedure closing the sofware frame
         * @param  e   a click on the top right box
         */
        public void actionPerformed(ActionEvent e) {
            moi.processWindowEvent(
                    new WindowEvent(moi, WindowEvent.WINDOW_CLOSING));
        }
    }
    
    
    /**
     * Implementation of Curves item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class curves_AL implements ActionListener {
        /**
         * Procedure opening the Curves Internal Frame
         * @param  e   a click on the Curves item
         */
        public void actionPerformed(ActionEvent e) {
            Curves curve = new Curves(moi,"");
            if (curve.dispose == false){
                curve.setVisible(true);
                desktop.add(curve);
                try {
                    curve.setSelected(true);
                } catch(java.beans.PropertyVetoException E) {
                }
            } else curve.dispose();
        }
    }
    
    
    /**
     * Implementation of Histograms item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class histo_AL implements ActionListener {
        /**
         * Procedure opening the Histograms Internal Frame
         * @param  e   a click on the Histograms item
         */
        public void actionPerformed(ActionEvent e) {
            
            Histograms histogram = new Histograms(moi,"");
            if (histogram.dispose == false){
                histogram.setVisible(true);
                desktop.add(histogram);
                try {
                    histogram.setSelected(true);
                } catch(java.beans.PropertyVetoException E) {
                }
            }
        }
    }
    
    
    /**
     * Implementation of Maps item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class maps_AL implements ActionListener {
        /**
         * Procedure opening the Maps Internal Frame
         * @param  e   a click on the Maps item
         */
        public void actionPerformed(ActionEvent e) {
            Graphiques map = new Maps(moi,"");
            //If map settings was well accomplished, map was setted visible so
            //map can be displayed
            if (map.dispose == false){
                map.setVisible(true);
                desktop.add(map);
                try {
                    map.setSelected(true);
                } catch(java.beans.PropertyVetoException E) {
                    E.printStackTrace();
                }
            }
        }
    }
    
    /**
     * Implementation of MapsAnim item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class mapsanim_AL implements ActionListener {
        /**
         * Procedure opening the Maps Internal Frame
         * @param  e   a click on the Maps item
         */
        public void actionPerformed(ActionEvent e) {
            MapsAnim map = new MapsAnim(moi,"");
            if (map.dispose == false){
                map.setVisible(true);
                desktop.add(map);
                try {
                    map.setSelected(true);
                } catch(java.beans.PropertyVetoException E) {
                }
            } else map.dispose();
        }
    }
    
    
    /**
     * Implementation of Contents item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class contents_AL implements ActionListener {
        
        /**
         * Procedure opening the Contents Internal Frame
         * @param  e   a click on the Contents item
         */
        public void actionPerformed(ActionEvent e) {
            
            HelpSet helpset = null;
            ClassLoader loader = MainFrame.class.getClassLoader();
            try{
                URL url = HelpSet.findHelpSet(loader, "help_helpset.hs");
                helpset = new HelpSet(null, url);
            } catch (Exception ev) {
                System.err.println("API Help Set not found");
            }
            HelpBroker help_content = helpset.createHelpBroker();
            help_content.setDisplayed(true);
            help_content.setFont(new Font("Dialog", Font.PLAIN, 12));
            help_content.setLocation(new Point(moi.getLocation().x + 20,
                    moi.getLocation().y + 50));
            help_content.setSize(new Dimension(moi.width - 40, moi.height));
            help_content.setCurrentID("welcome");
        }
    }
    
    
    /**
     * Implementation of About item listener
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class about_AL implements ActionListener {
        
        /**
         * Procedure opening the About Internal Frame
         * @param  e   a click on the About item
         */
        public void actionPerformed(ActionEvent e) {
            // ouvre une fenetre avec nom auteur, labo, et autre
            /*JOptionPane.showInternalMessageDialog(moi.getContentPane(),
                    "TESD 1.0\n"
                    + "UMR CNRS 5558 - \"BBE\"\n"
                    + "Grgory Deceliere\n"
                    + "Yann Letrillard",
                    "About TESD",
                    JOptionPane.PLAIN_MESSAGE);*/
            JInternalFrame about = new JInternalFrame("About TESD", true, true,true,true);
            about.setVisible(true);
            about.setLocation(new Point(5,5));
            about.setSize(600,460);
            about.setResizable(false);
            JEditorPane text_pane = new JEditorPane();
            desktop.add(about);
            about.getContentPane().add(text_pane, BorderLayout.CENTER);
            
            try {
                URL about_url = MainFrame.class.getResource("images/About.html");
                text_pane.setPage(about_url);
                about.setSelected(true);
            } catch (IOException ex) {
                text_pane.setText(ex.toString());
            }catch(java.beans.PropertyVetoException E) {
            }
        }
    }
    
    /**
     * Propose to the user to save the simualtions parameters in
     * a .para file
     * @throws java.io.IOException If theres a mistake
     */
    protected void saveInputFile()throws IOException{
        
        String message = "Do you want to save these simulation parameters in a .para file?";
        Object[] options = {"Yes", "No"};
        int choice = JOptionPane.showOptionDialog(moi, message,
                "", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options,options[1]);
        
        if (choice == 0){
            /*JFileChooser chooser = new JFileChooser();
            chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            chooser.setApproveButtonText("Ok");
            chooser.setDialogTitle("Choose a directory to save the .para file");
             
            int return_value = chooser.showDialog(moi, null);
            if (return_value == chooser.APPROVE_OPTION) {
                String outputFileName = chooser.getSelectedFile().toString() + System.getProperty("file.separator") + this.model.nomFic2 + ".para";
                File outPutFile = new File(outputFileName);
                if (outPutFile.exists()){
                    String message2 = "You already gave the name:" + this.model.nomFic2 + "to a simulation\n" +
                            "Enter a new name:";
                    JOptionPane.showInternalInputDialog(this, message2, "simulation renaming", JOptionPane.QUESTION_MESSAGE, null, null, null);
             
                } else {
                    this.setCurrentPath(chooser.getSelectedFile().toString());
             
                    try {
                        File file = new File(chooser.getSelectedFile().toString() + System.getProperty("file.separator")
                        + this.model.nomFic2.toString() + ".para");
                        this.model.nomFic2 = file.toString();
                        OutPutDataFiles.saveParameters(this.model, file);
                    } catch(IOException io_exception) {
                        io_exception.printStackTrace();
                    }
                }
             
            }*/
            try {
                OutPutDataFiles.saveParameters(this.model, new File(this.model.nomFic1 + ".para"));
            } catch(IOException io_exception) {
                io_exception.printStackTrace();
            }
            SimulationForm next_form = new SimulationForm(moi, true, moi);
            int next_formX = (int)this.getBounds().getCenterX()-(next_form.getWidth()/2);
            int next_formY = (int)this.getBounds().getCenterY()-(next_form.getHeight()/2);
            next_form.setLocation(new Point(next_formX, next_formY));
            next_form.setVisible(true);
            
        } else if (choice == 1){
            Simulation uneSimule = new Simulation(moi);
            moi.desktop.add(uneSimule);
            
            try {
                uneSimule.setSelected(true);
            } catch (java.beans.PropertyVetoException ve) {
            }
        }
        
    }
    
}




