package projet_these.ig;

import projet_these.et.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.beans.*;




/**
 * Implementation of a general internal frame used
 * to ask some questions
 * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
 * @version 01/03/02
 */
public class SimpleQuestion extends JInternalFrame {
    
    
    /**
     * "Choose your model" question
     */
    protected JLabel L1;
    
    /**
     * The Ok button
     */
    protected JButton B1;
    
    /**
     * The return button
     */
    protected JButton B2;
    
    /**
     * The cancel button
     */
    protected JButton B3;
    
    
    /**
     * TextField to get the number of generations
     */
    protected JTextField T1;
    
    /**
     * The panel for buttons
     */
    protected JPanel panel1;
    
    // Reference de l'instance courante
    /**
     * This frame
     */
    protected SimpleQuestion moi;
    
    /**
     * The TESD main frame
     */
    protected MainFrame super_frame;
    
    /**
     * The question
     */
    private String question;
    
    /**
     * The actual font
     */
    private Font f;
    
    /**
     * Used when questions have to be asked several times
     */
    protected static int compteur = 0;
    
    
    /**
     * Constructor of initialization of SimpleQuestion
     * @param super_frame The main frame
     * @param q           The question asked
     */
    public SimpleQuestion(MainFrame super_frame, String q) {
        
        /* Constructeur d'une JInternalFrame sans titre, non redimensionnable,
        fermable, non maximisable, non iconifiable*/
        super("", false, true, false, false);
        
        // Recuperation d'une reference sur la fenetre principale
        setsuperFrame(super_frame);
        
        // La fenetre interne courante
        moi = this;
        
        // La question posee dans cette fenetre
        question = q;
        
        // Utilise comme couleur de font celle de super_frame.mb
        setBackground(getsuperFrame().getJMenuBar().getBackground());
        
        // Gestion de la fermeture de la fenetre
        addInternalFrameListener(new Qsimpl_adapter());
        
        // Creation d'une police
        f = new Font("Dialog", Font.PLAIN, 12);
        
        // Gestion de la mise en page par un GridBagLayout
        getContentPane().setLayout(new GridBagLayout());
        
        // Creation et positionnnement de L1
        L1 = new JLabel(question);
        L1.setFont(f);
        util_IG.addComposantContentPane(getContentPane(), L1,
                GridBagConstraints.EAST, 0, 0, 1, 1,
                new Insets(10, 0, 0, 10), 0, 0);
        
        // Creation et positionnement de T1
        T1 = new JTextField();
        util_IG.addComposantContentPane(getContentPane(), T1,
                GridBagConstraints.WEST, 1, 0, 1, 1,
                new Insets(10, 0, 0, 0), 150, 0);
        
        // Creation du bouton OK
        B1 = new JButton("OK");
        B1.setFont(f);
        
        // Creation du bouton Return
        B2 = new JButton("Return");
        B2.setFont(f);
        
        // Creation du bouton Cancel
        B3 = new JButton("Cancel");
        B3.setFont(f);
        
        // Gestion de la fermeture par activation de Cancel
        B3.addActionListener(new cancel_AL());
        
        // Creation d'un panneau recevant les boutons OK et Cancel
        panel1 = new JPanel();
        panel1.setBackground(getContentPane().getBackground());
        panel1.add(B1);
        panel1.add(B2);
        panel1.add(B3);
        util_IG.addComposantContentPane(getContentPane(), panel1,
                GridBagConstraints.CENTER, 1, 1, 1, 1,
                new Insets(0, 0, 10, 0), 0, 0);
        
        // Impossible de redimensionner la fenetre
        setResizable(false);
        
        // La fenetre doit etre visible
        setVisible(true);
    }
    
    
    /**
     * Frame's Procedure of positionning
     */
    protected void setup() {
        
        // Largeur optimale du JLabel
        int xL = L1.getPreferredSize().width;
        
        // Hauteur optimale du JLabel
        int yL = L1.getPreferredSize().height;
        
        // Largeur optimale du JTextField
        int xT = T1.getPreferredSize().width;
        
        // Hauteur optimale du JTextField
        int yT = T1.getPreferredSize().height;
        
        // Largeur optimale du JPanel
        int xpanel = panel1.getPreferredSize().width;
        
        // Hauteur optimale du JPanel
        int ypanel = panel1.getPreferredSize().height;
        
        // Uniformisation des Dimension de chaque composant
        L1.setPreferredSize(new Dimension(xL, yL));
        L1.setMaximumSize(new Dimension(xL, yL));
        L1.setMinimumSize(new Dimension(xL, yL));
        T1.setPreferredSize(new Dimension(xT, yT));
        T1.setMaximumSize(new Dimension(xT, yT));
        T1.setMinimumSize(new Dimension(xT, yT));
        panel1.setPreferredSize(new Dimension(xpanel, ypanel));
        panel1.setMaximumSize(new Dimension(xpanel, ypanel));
        panel1.setMinimumSize(new Dimension(xpanel, ypanel));
        
        // Dimensionnement de la fenetre
        int x = Math.max(xL + xT, xL + xpanel);
        int y = Math.max(yL, yT);
        y = y + ypanel;
        setSize((int)((x + 15) * 1.2), (int)((y + 15) * 1.2));
        
        // Positionnement de la fenetre
        setLocation(10, 10);
    }
    
    
    /**
     * Frame's Procedure of centering
     * @param   c1  The question
     * @param   c2  The answer
     * @param   c3  The buttons OK and Cancel
     */
    protected void allCentered(JComponent c1, JComponent c2, JComponent c3) {
        
        // On enleve tout pour presenter la fenetre differemment
        getContentPane().removeAll();
        
        // Repositionnement de c1
        util_IG.addComposantContentPane(getContentPane(), c1,
                GridBagConstraints.CENTER, 0, 0, 1, 1,
                new Insets(10, 10, 5, 10), 0, 0);
        
        // Repositionnement de c2
        util_IG.addComposantContentPane(getContentPane(), c2,
                GridBagConstraints.CENTER, 0, 1, 1, 1,
                new Insets(5, 0, 5, 0), 170, 0);
        
        // Repositionnement de c3
        util_IG.addComposantContentPane(getContentPane(), c3,
                GridBagConstraints.CENTER, 0, 2, 1, 1,
                new Insets(5, 0, 5, 0), 0, 0);
        
        // Dimensionnement et positionnement
        setSize(getPreferredSize());
        this.setLocation(10, 10);
    }
    
    
    /**
     * Procedure returning a reference to main frame
     * @return the main frame
     */
    protected MainFrame getsuperFrame() {
        
        return super_frame;
    }
    
    
    /**
     * Procedure setting the reference to main frame
     * @param   sf  The main frame
     */
    protected void setsuperFrame(MainFrame sf) {
        
        super_frame = sf;
    }
    
    
    /**
     * Implementation of the ActionListeneter interface
     * to listen to the events of the components
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 01/03/02
     */
    private class cancel_AL implements ActionListener {
        
        
        /**
         * Procedure closing the SimpleQuestion frame
         * @param  e   a click on the Cancel button
         */
        public void actionPerformed(ActionEvent e) {
            try {
                moi.setClosed(true);
            } catch (PropertyVetoException exc) {
            };
        }
    }
    
    
    /**
     * Implementation of the main software frame adapter
     * @author Gregory Deceliere - UMR CNRS 5558 - "BBE"
     * @version 12/12/02
     */
    private class Qsimpl_adapter extends InternalFrameAdapter {
        
        /**
         * Procedure closing the Question Simple frame
         * @param  e   a click on the top right box
         */
        public void internalFrameClosing(InternalFrameEvent e) {
            
            // Reactivation des menus Simulations
            getsuperFrame().menu.setEnabled(true);
            
            // Reinitialisation du compteur
            compteur = 0;
            
            // Fermeture de la fenetre SimpleQuestion
            dispose();
        }
    }
}



