package projet_these.ig;

import java.awt.*;
import projet_these.et.*;
import javax.swing.*;




/**
 * Titre : util_IG
 * Description : utilitaires de construction d'interfaces graphiques
 * Societe : UMR 5558
 * @author Gregory Deceliere
 * @version 01/03/02
 */
class util_IG
{

    /**
     * Code associated to a positive float
     */
    protected final static int POS_FLOAT0 = 0;
    
    /**
     * Code associated to a trictly positive float
     */
    protected final static int POS_FLOAT = 1;
    
    /**
     * Code associated to an integer superior to 2
     */
    protected final static int POS_INT = 2;
    
    /**
     * Procedure adding a component to the rootPane
     * @param rootPane   the rootPane receiving the component
     * @param composant  the component to add
     * @param anchor     the position in his display area
     * @param gridx      the cell at the left of the component's display area
     * @param gridy      the cell at the top of the component's display area
     * @param gridwidth  the number of cells in a row for the component's
     *				display area
     * @param gridheight the number of cells in a column for the component's
     *                          display area
     * @param insets     the external padding of the component
     * @param ipadx      the internal padding of the component
     * @param ipady      the internal padding of the component
     */

    protected static void addComposantContentPane(Container rootPane,
            Component composant, int anchor, int gridx, int gridy,
            int gridwidth, int gridheight, Insets insets, int ipadx,
            int ipady)
    {
        
        //Constraints creation for component management by GridBad
        GridBagConstraints c = new GridBagConstraints();
        
        // In its area positionning's component
        c.anchor = anchor;
        
        // x coordinates of the most left component
        c.gridx = gridx;
        
        // x coordinates of the upper component
        c.gridy = gridy;
        
        // Width block in cells number
        c.gridwidth = gridwidth;
        
        // Height block in cells number
        c.gridheight = gridheight;
        
        // Insets around components
        c.insets = insets;
        
        // Increase over minimum size
        c.ipadx = ipadx;
        c.ipady = ipady;
        
        // The component is added to contentPane
        rootPane.add(composant, c);
    }
    
    
    /**
     * Procedure sending sending error message
     * @return true if an error message has been sent
     * @param int_fr JInternalFrame the frame
     * @param value String
     * @param test int
     */
    protected static boolean errorManager(JInternalFrame int_fr,
            String value, int test)
    {
        
        // String for error message
        String message = "";
        
        // Main frame
        JFrame fr = new JFrame();
        if (int_fr.getClass().getName() ==
                "projet_these.ig.QuestionSimple" ||
                int_fr.getClass().getName() ==
                "projet_these.ig.PopulationSizes" ||
                int_fr.getClass().getName() ==
                "projet_these.ig.CopiesNumbers" ||
                int_fr.getClass().getName() ==
                "projet_these.ig.MigrationMatrix1" ||
                int_fr.getClass().getName() ==
                "projet_these.ig.MigrationMatrix2")
            fr = ((SimpleQuestion) int_fr).getsuperFrame();
        
        // Appropriate error management selection
        switch (test)
        {
            case POS_FLOAT0 :
                // If answer is absent from allowed answers, error signaling
                if (! Util_Islands.isPositiveFloat(value))
                {
                    message =
                            "Error of data entry\n" +
                            "Enter a positive float value, or deselect cells";
                    JOptionPane.showMessageDialog(fr, message, "", JOptionPane.ERROR_MESSAGE);
                    return true;
                };
                break;
                
            case POS_FLOAT :
                // If answer is absent from allowed answers, error signaling
                if (! Util_Islands.isPositiveFloat(value) || value.equals(""))
                {
                    message =
                            "Error of data entry\n" +
                            "Enter a strict positive float value or deselect cells";
                    JOptionPane.showMessageDialog(fr, message, "", JOptionPane.ERROR_MESSAGE);
                    return true;
                };
                break;
                
            case POS_INT :
                // If answer is absent from allowed answers, error signaling
                if (! Util_Islands.isNumeric(value) || value.equals("") ||
                        ! (Integer.valueOf(value).intValue() > 1))
                {
                    message =
                            "Error of data entry\n" +
                            "Enter an integer value higher than one";
                    JOptionPane.showMessageDialog(fr, message, "", JOptionPane.ERROR_MESSAGE);
                    return true;
                }
                break;
        }
        
        return false;
    }
    
}

